/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to You under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.apache.webbeans.test.unittests.typedliteral;

import java.lang.annotation.Annotation;
import java.util.List;

import javax.inject.AnnotationLiteral;
import javax.inject.Current;
import javax.inject.TypeLiteral;
import javax.inject.manager.Bean;

import junit.framework.Assert;

import org.apache.webbeans.component.AbstractComponent;
import org.apache.webbeans.container.ManagerImpl;
import org.apache.webbeans.context.ContextFactory;
import org.apache.webbeans.test.component.ITypeLiteralComponent;
import org.apache.webbeans.test.component.InjectedTypeLiteralComponent;
import org.apache.webbeans.test.component.TypeLiteralComponent;
import org.apache.webbeans.test.servlet.TestContext;
import org.junit.Before;
import org.junit.Test;

public class TypedLiteralComponentTest extends TestContext
{
    public TypedLiteralComponentTest()
    {
        super(TypedLiteralComponentTest.class.getSimpleName());
    }

    @Before
    public void init()
    {
        super.init();
    }

    @Test
    public void testTypedComponent() throws Throwable
    {
        clear();

        defineSimpleWebBean(TypeLiteralComponent.class);
        defineSimpleWebBean(InjectedTypeLiteralComponent.class);
        List<AbstractComponent<?>> comps = getComponents();

        ContextFactory.initRequestContext(null);

        Assert.assertEquals(2, comps.size());

        TypeLiteralComponent userComponent = (TypeLiteralComponent) getManager().getInstance(comps.get(0));
        InjectedTypeLiteralComponent tc = (InjectedTypeLiteralComponent) getManager().getInstance(comps.get(1));

        Assert.assertNotNull(tc.getComponent());
        Assert.assertNotNull(userComponent);

        Assert.assertTrue(tc.getComponent() instanceof TypeLiteralComponent);
        
        Assert.assertNotNull(tc.getEraseComponent());

        Assert.assertEquals("GURKAN", TypeLiteralComponent.STR);

        ContextFactory.destroyRequestContext(null);
    }

    @Test
    public void testTypedLiteralComponent() throws Throwable
    {
        clear();

        defineSimpleWebBean(TypeLiteralComponent.class);
        List<AbstractComponent<?>> comps = getComponents();

        ContextFactory.initRequestContext(null);

        Assert.assertEquals(1, comps.size());

        TypeLiteral<ITypeLiteralComponent<List<String>>> tl = new TypeLiteral<ITypeLiteralComponent<List<String>>>()
        {
        };

        Annotation[] anns = new Annotation[1];
        anns[0] = new AnnotationLiteral<Current>()
        {

        };

        Bean<ITypeLiteralComponent<List<String>>> s = ManagerImpl.getManager().resolveByType(tl, anns).iterator().next();
        Assert.assertNotNull(s);

        ContextFactory.destroyRequestContext(null);
    }

}
